/* eslint-env jest */

const { runPlugins } = require('./helpers');

const varPlugin_ = require('../var');
const varPlugin = x =>
  varPlugin_(x, { transform: x => `token(${x})` });

it('replaces valid token within single fallback', () => {
  const actual = runPlugins(
    `
    background: var(--sds-c-icon-color-background, $color-brand);
  `,
    [varPlugin(['color-brand'])]
  );
  expect(actual).toMatchSnapshot();
});

it('ignores non-token variable within single fallback', () => {
  const actual = runPlugins(
    `
    @each $name, $color in $bg-actions-map {

      .slds-icon-#{$name} {
        background-color: var(--sds-c-icon-color-background, $color);
      }
    }
  `,
    [varPlugin([])]
  );
  expect(actual).toMatchSnapshot();
});

it('replaces valid tokens within double fallback', () => {
  const actual = runPlugins(
    `
    background: var(--sds-c-icon-color-background, var(--sds-g-icon-color-background, $color-brand));
  `,
    [varPlugin(['color-brand'])]
  );
  expect(actual).toMatchSnapshot();
});

it('replaces valid token within multiple css var statements', () => {
  const actual = runPlugins(
    `
    padding: var(--sds-c-button-padding-vertical, $spacing-small) var(--sds-c-button-padding-horizontal, $spacing-large);
  `,
    [varPlugin(['spacing-small', 'spacing-large' ])]
  );
  expect(actual).toMatchSnapshot();
});

it('replaces valid token within css var assignment with var function value', () => {
  const actual = runPlugins(
    `
    .a { --sds-c-icon-color-background: var(--sds-c-icon-color-background, #{$color-brand}); }
  `,
    [varPlugin(['color-brand'])]
  );
  expect(actual).toMatchSnapshot();
});

it('replaces valid token within css var assignment with var function value with other interpolations', () => {
  const actual = runPlugins(
    `
    .a { --sds-c-button-spacing-horizontal-start: var(--sds-c-button-#{$api-variant}-spacing-horizontal-start, #{$spacing-medium}); }
  `,
    [varPlugin(['spacing-medium'])]
  );
  expect(actual).toMatchSnapshot();
});