const _ = require('lodash');
const MagicString = require('magic-string');
const createQueryWrapper = require('query-ast');
const { stringify, parse } = require('scss-parser');

const TOKENIZEABLE_FUNCTIONS = [
  'var'
];

module.exports = (tokens, options = {}) => $ => {
  const wrapToken = n => {
    const fnName = n.node.value[0].value;
    // Only tokenize CSS functions
    if (!TOKENIZEABLE_FUNCTIONS.includes(fnName)) return n;

    const variables = $(n)
      .children('arguments')
      .find('variable')
      .get();
    // Skip functions with no tokens
    if (!variables.length) return n;
    // Make sure any variables used in the function can be tokenized
    if (!variables.filter(v => _.includes(tokens, v.value)).length) return n;


    // In cases of css var assignment with var function value, the var is wrapped in an interpolation
    // i.e. .a { --sds-c-icon-color-background: var(--sds-c-icon-color-background, #{$color-brand}); }
    // We want to remove the interpolation to wrap it with token()
    $(n)
      .children('arguments')
      .children('interpolation')
      .replace(n => {
        const { type, value } = n.node.value[0];
        if (type === 'variable' && _.includes(tokens, value)) {
          return n.node.value[0];
        }
        return n;
      });

    const valueString = stringify(n.toJSON());
    const valueStringMagic = new MagicString(valueString);
    const $$ = createQueryWrapper(parse(valueString));

    // The styling api assignments will come in this format:
    // padding-right: var(--sds-c-breadcrumbs-spacing-horizontal-end, $spacing-x-small);
    // This plugin is very similar to the function plugin, but keeping isolated as separate plugin, jic
    $$('function')
      .children('arguments')
      .children('variable')
      .get()
      .map(v => {
        const start = v.start.cursor;
        const end = v.next.cursor;
        const name = _.camelCase(valueString.slice(start + 1, end));
        return { start, end, name };
      })
      .forEach(v => {
        valueStringMagic.overwrite(v.start, v.end, `"+#{${v.name}}+"`);
      });
    return parse(options.transform(`"${valueStringMagic.toString()}"`))
      .value[0];
  };
  $()
    .find('function')
    .replace(wrapToken);
};
